package com.huimv.apiservice.service.impl;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huimv.apiservice.dao.PigDao;
import com.huimv.apiservice.dao.PigstyDao;
import com.huimv.apiservice.dao.YearPigHistoryImgDao;
import com.huimv.apiservice.dao.YearPigPedigreeDao;
import com.huimv.apiservice.entity.EmployeeEntity;
import com.huimv.apiservice.entity.PigstyEntity;
import com.huimv.apiservice.entity.YearPigBaseEntity;
import com.huimv.apiservice.entity.YearPigPedigreeEntity;
import com.huimv.apiservice.dao.*;
import com.huimv.apiservice.entity.*;
import com.huimv.apiservice.entity.vo.*;
import com.huimv.apiservice.service.PigService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Author yinhao
 * @Date 2021/5/8 17:22
 * @Description
 */
@Service
public class PigServiceImpl extends ServiceImpl<PigDao, YearPigBaseEntity> implements PigService {

    @Autowired
    private PigstyDao pigstyDao;

    @Autowired
    private YearPigHistoryImgDao yearPigHistoryImgDao;

    @Autowired
    private YearPigPedigreeDao yearPigPedigreeDao;

    @Autowired
    private SleepStatusDao sleepStatusDao;

    @Autowired
    private SportTimeDao sportTimeDao;

    @Override
    public PigBaseVo getPigInfoByEarTagNo(String pigEarTagNo) {

        LambdaQueryWrapper<YearPigBaseEntity> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(YearPigBaseEntity::getEartag, pigEarTagNo);

        YearPigBaseEntity yearPigBaseEntity = baseMapper.selectOne(lambdaQueryWrapper);


        PigBaseVo pigBaseVo = new PigBaseVo();
//        BeanUtil.copyProperties(yearPigBaseEntity,pigBaseVo);
        if (ObjectUtil.isNotNull(yearPigBaseEntity)) {
            pigBaseVo.setWeight(yearPigBaseEntity.getWeight());
            pigBaseVo.setDayAge(yearPigBaseEntity.getDayAge());
            pigBaseVo.setBreed(yearPigBaseEntity.getBreed());
            //虚拟数据 待填充
            pigBaseVo.setBirthday(new Date());
            Integer unitId = yearPigBaseEntity.getUnitId();
            PigstyEntity pigstyEntity = null;
            if (unitId != null) {
                 pigstyEntity = pigstyDao.selectById(unitId);
            }

            if (ObjectUtil.isNotNull(pigstyEntity)) {
                String number = pigstyEntity.getNumber();
                if (StringUtils.isNotEmpty(number)) {
                    pigBaseVo.setPigsty(number);
                }
            }
            //虚拟数据 待填充
            pigBaseVo.setBodyTemperature("100");
        }
        return pigBaseVo;
    }

    @Override
    public PigImageVo getImageByEarTagNo(String pigEarTagNo) {

        PigImageVo pigImageVo = new PigImageVo();

        pigImageVo.setPigEarTagNo(pigEarTagNo);
        String img = yearPigHistoryImgDao.getLatestImgByPigEarTagNo(pigEarTagNo);
        if (StringUtils.isNotEmpty(img)) {
            //String encodeImg = Base64.encode(img);
            pigImageVo.setImageData(img);
        }
        return pigImageVo;
    }

    @Override
    public List<PigBaseVo> getListByBreed(String breedName) {
        LambdaQueryWrapper<YearPigBaseEntity> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(YearPigBaseEntity::getBreed, breedName);
        List<YearPigBaseEntity> yearPigBaseEntityList = baseMapper.selectList(lambdaQueryWrapper);

        List<PigBaseVo> pigBaseVoList = new ArrayList<>();
        if (CollUtil.isNotEmpty(yearPigBaseEntityList)) {
            for (YearPigBaseEntity yearPigBaseEntity : yearPigBaseEntityList) {
                PigBaseVo pigBaseVo = new PigBaseVo();
                pigBaseVo.setWeight(yearPigBaseEntity.getWeight());
                pigBaseVo.setDayAge(yearPigBaseEntity.getDayAge());
                pigBaseVo.setBreed(yearPigBaseEntity.getBreed());
                //虚拟数据 待填充
                pigBaseVo.setBirthday(new Date());
                Integer unitId = yearPigBaseEntity.getUnitId();
                PigstyEntity pigstyEntity = null;
                if (unitId != null) {
                    pigstyEntity = pigstyDao.selectById(unitId);
                }
                if (ObjectUtil.isNotNull(pigstyEntity)) {
                    String number = pigstyEntity.getNumber();
                    if (StringUtils.isNotEmpty(number)) {
                        pigBaseVo.setPigsty(number);
                    }
                }
                //虚拟数据 待填充
                pigBaseVo.setBodyTemperature("100");
                pigBaseVoList.add(pigBaseVo);
            }
        }
        return pigBaseVoList;
    }

    @Override
    public PigPedigreeVo getPigPedigree(String pigEarTagNo) {

        LambdaQueryWrapper<YearPigBaseEntity> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(YearPigBaseEntity::getEartag, pigEarTagNo);

        YearPigBaseEntity yearPigBaseEntity = baseMapper.selectOne(lambdaQueryWrapper);
        PigPedigreeVo pigPedigreeVo = new PigPedigreeVo();

        if (ObjectUtil.isNotNull(yearPigBaseEntity)) {
            pigPedigreeVo.setBreed(yearPigBaseEntity.getBreed());
            //虚拟数据 待填充
            pigPedigreeVo.setBirthday(new Date());

            LambdaQueryWrapper<YearPigPedigreeEntity> lambdaQuery = Wrappers.lambdaQuery();
            lambdaQuery.eq(YearPigPedigreeEntity::getPigletEartag, pigEarTagNo);
            YearPigPedigreeEntity yearPigPedigreeEntity = yearPigPedigreeDao.selectOne(lambdaQuery);

            String motherEarTag = yearPigPedigreeEntity.getSowEartag();
            String fatherEarTag = yearPigPedigreeEntity.getBoarEartag();

            lambdaQueryWrapper.clear();
            lambdaQueryWrapper.eq(YearPigBaseEntity::getEartag, motherEarTag);
            YearPigBaseEntity mother = baseMapper.selectOne(lambdaQueryWrapper);
            PigPedigreeMotherVo pigPedigreeMotherVo = new PigPedigreeMotherVo();
            if (ObjectUtil.isNotNull(mother)) {
                pigPedigreeMotherVo.setBreed(mother.getBreed());
                pigPedigreeMotherVo.setWeight(mother.getWeight());
                //虚拟数据 待填充
                pigPedigreeMotherVo.setPregnancyDate(new Date());
                pigPedigreeMotherVo.setChildCount(10);
                pigPedigreeMotherVo.setOriginPlace("产地信息");
            }
            pigPedigreeVo.setMother(pigPedigreeMotherVo);

            lambdaQuery.clear();
            lambdaQueryWrapper.eq(YearPigBaseEntity::getEartag, fatherEarTag);
            PigPedigreeFatherVo pigPedigreeFatherVo = new PigPedigreeFatherVo();
            YearPigBaseEntity father = baseMapper.selectOne(lambdaQueryWrapper);
            if (ObjectUtil.isNotNull(father)) {
                pigPedigreeFatherVo.setBreed(father.getBreed());
                //虚拟数据 待填充
                pigPedigreeFatherVo.setOriginPlace("产地信息");
            }
            pigPedigreeVo.setFather(pigPedigreeFatherVo);
        }

        return pigPedigreeVo;
    }

    @Override
    public PigKeeperVo getKeeper(String pigEarTagNo) {

        PigKeeperVo pigKeeperVo = new PigKeeperVo();
        pigKeeperVo.setPigEarTagNo(pigEarTagNo);

        EmployeeEntity employeeEntity = baseMapper.selectFeederInfoByPigEarTagNo(pigEarTagNo);
        FeederVo feederVo = pigKeeperVo.getFeeder();
        if (employeeEntity != null) {
            feederVo.setName(employeeEntity.getName());
            feederVo.setPosition(employeeEntity.getPosition());
            feederVo.setApproachTime(employeeEntity.getApproachTime());
            feederVo.setDepartureTime(employeeEntity.getDepartureTime());
        }

        return pigKeeperVo;
    }

    @Override
    public PigSportVo getActivityAmount(String pigEarTagNo) {
        PigSportVo sportVo = new PigSportVo();

        LambdaQueryWrapper<SportTimeEntity> lambdaQueryWrapper = Wrappers.lambdaQuery();

        //虚假数据
        lambdaQueryWrapper.eq(SportTimeEntity::getEartag,pigEarTagNo);
        SportTimeEntity sportTimeEntity = sportTimeDao.selectOne(lambdaQueryWrapper);
        sportVo.setActivityAmount(sportTimeEntity.getSportCount());
        sportVo.setPigEartagNo(pigEarTagNo);
        return sportVo;
    }

    @Override
    public SleepStatusEntity getSleepStatus(String pigEarTagNo) {

        //虚假数据
        LambdaQueryWrapper<SleepStatusEntity> lambdaQueryWrapper = Wrappers.lambdaQuery();
        lambdaQueryWrapper.eq(SleepStatusEntity::getEartag,pigEarTagNo);
        SleepStatusEntity sleepStatusEntity = sleepStatusDao.selectOne(lambdaQueryWrapper);


        return sleepStatusEntity;
    }

    @Override
    public EnvVo getIndoorEnv(String pigEarTagNo) {

        return null;
    }
}
